#
#  This file is part of GroupKit
#
#  (c) Copyright 1992 Department of Computer Science, University of
#      Calgary, Calgary, Alberta, Canada.  All rights reserved.
#
#  Permission to use, copy, modify, and distribute this software and its
#  documentation for any purpose and without fee is hereby granted, provided
#  that the above copyright notice appears in all copies.  The University
#  of Calgary makes no representations about the suitability of this
#  software for any purpose.  It is provided "as is" without express or
#  implied warranty.
#

# Written by David Marwood

# 
# The non-optimistic lock waits when GrabLock is called until approval
# is received from the other sites.
# 

# ----------------------------------------------------------------
# External Data
# ----------------------------------------------------------------
set LockLayerName "non-optimistic"

# A list of all the locks held (approved).
set HeldLocks {}


# ----------------------------------------------------------------
# Internal Data
# ----------------------------------------------------------------

# GotNetResponse: Set to No by GrabLock.  Set to Yes by Network
# response.
set GotNetResponse No


# ----------------------------------------------------------------
# Creating locks.
# ----------------------------------------------------------------

# 
# Creates a lock, LockName.
# 
proc CreateLock LockName {
    global LockStates
    Assert "[info exists LockStates($LockName)] == 0"
    set LockStates($LockName) Released
    gk_toOthers RemoteCreateLock $LockName
    NetworkCreateLock $LockName
}


# 
# When a lock is created, RemoteCreateLock is called at all the sites
# except the creator site, where CreateLock is called.
# 
proc RemoteCreateLock LockName {
    global LockStates
    set LockStates($LockName) Released
}


# 
# A trace on the gk_entrantUserNum to detect when a new user arrives.  If
# this site is responsible for updating the new user, then all the
# existing locks are created on the new site.
# 
trace variable gk_entrantUserNum w LockNewUser
proc LockNewUser args {
#    puts "LockNewUser:"
    global gk_entrantUserNum
    global LockStates
    global new_user
    foreach Name [array names LockStates] {
	gk_toUserNum $gk_entrantUserNum RemoteCreateLock $Name
    }
}


# ----------------------------------------------------------------
# Grabbing/releasing locks.
# ----------------------------------------------------------------

# 
# Calls NetworkRequest and waits for a response.  LockName must be
# released.  Returns LockStates($LockName).  mytkwait is used to
# prevent X events from getting through while waiting for the response
# from the other site.  This is intended to prevent button releases or
# other user action.  Really, it should stop all processing until the
# lock is approved, but that would prevent the approval message from
# getting through.
# 
proc GrabLock LockName {
    global GotNetResponse 
    set GotNetResponse No

#    puts "GrabLock: Starting NetworkRequest."
    NetworkRequest $LockName
#    puts "GrabLock: Waiting for net response."
    while {$GotNetResponse == "No"} {
	if {[info commands mytkwait] == ""} {
	    tkwait variable GotNetResponse
	} else {
	    mytkwait variable GotNetResponse 4
	}
    }
    global LockStates
#    puts "GrabLock: Done."
    return LockStates($LockName)
}


# 
# Releases the lock LockName.  LockName must be held (approved).
# 
proc ReleaseLock LockName {
#    puts "ReleaseLock:"
    global HeldLocks
    Remove HeldLocks $LockName

    global LockStates
    set LockStates($LockName) Released
}


# ----------------------------------------------------------------
# Procedures called by network layer.
# ----------------------------------------------------------------

# 
# Called by the network layer in response to the NetworkRequest call.
# Sets LockStates(LockName) to Approved or Denied, according to the
# response.
# 
proc NetworkResponse {LockName YesNo} {
#    puts stdout "NetworkResponse: lock = $LockName, yesno = $YesNo"
    global GotNetResponse
    set GotNetResponse 1

    global LockStates
    if {$YesNo == "Yes"} {
	global HeldLocks
	Insert HeldLocks $LockName
#	puts "Trace info for LockState($LockName): [trace vinfo LockStates($LockName)]"
	set LockStates($LockName) Approved
#	puts "NetworkResponse: Lock became approved"
    } else {
	set LockStates($LockName) Denied
#	puts "NetworkResponse: Lock became denied"
    }
}


