#
#  This file is part of GroupKit
#
#  (c) Copyright 1992 Department of Computer Science, University of
#      Calgary, Calgary, Alberta, Canada.  All rights reserved.
#
#  Permission to use, copy, modify, and distribute this software and its
#  documentation for any purpose and without fee is hereby granted, provided
#  that the above copyright notice appears in all copies.  The University
#  of Calgary makes no representations about the suitability of this
#  software for any purpose.  It is provided "as is" without express or
#  implied warranty.
#

# Written by David Marwood

# 
# A semi-optimistic lock can be grabbed without waiting, but must be
# approved before releasing.  This lock waits until the tentative
# state becomes approved before releasing.
#
# Requires sourcing of a network layer before this file.
# 

# ----------------------------------------------------------------
# External data structures
# ----------------------------------------------------------------

# HeldLocks is a list of held (approved) locks.
set HeldLocks {}

set LockLayerName "semi-optimistic"

# ----------------------------------------------------------------
# Creating a lock.
# ----------------------------------------------------------------

# 
# Creates the lock LockName. Must be called before GrabLock or
# ReleaseLock.
# 
proc CreateLock LockName {
    global LockStates
    Assert "[info exists LockStates($LockName)] == 0"
    set LockStates($LockName) Released
    gk_toOthers RemoteCreateLock $LockName
    global new_user
    NetworkCreateLock $LockName
}


# 
# A trace on the gk_entrantUserNum to detect when a new user arrives.  If
# this site is responsible for updating the new user, then all the
# existing locks are created on the new site.
# 
trace variable gk_entrantUserNum w LockNewUser
proc LockNewUser args {
#    puts "LockNewUser:"
    global LockStates
    global gk_entrantUserNum
    foreach Name [array names LockStates] {
	gk_toUserNum $gk_entrantUserNum RemoteCreateLock $Name
    }
#    puts "LockNewUser: Done"
}


# 
# When a lock is created, RemoteCreateLock is called at all the sites
# except the creator site, where CreateLock is called.
# 
proc RemoteCreateLock LockName {
    global LockStates
    set LockStates($LockName) Released
}


# ----------------------------------------------------------------
# Grabbing/releasing locks.
# ----------------------------------------------------------------

# 
# Attempts to grab lock LockName.  LockName must be released or
# denied.  Calls NetworkRequest and returns Tentative.
# 
proc GrabLock LockName {
    global LockStates
    Assert "[info exists LockStates($LockName)] == 1"
    Assert "[string compare $LockStates($LockName) Released] == 0 || [string compare $LockStates($LockName) Denied] == 0"

    set LockStates($LockName) Tentative

    global HeldLocks
    Insert HeldLocks $LockName

    # puts "GrabLock: Starting NetworkRequest."
    NetworkRequest $LockName
    return $LockStates($LockName)
}


# 
# Releases lock LockName.  LockName must be held (approved).  If
# LockName is tentative then ReleaseLock tkwaits until it is approved.
# Really, it should stop all processing until the lock is released,
# but then the approval message wouldn't get through.
# 
proc ReleaseLock LockName {
    global LockStates
    
    Assert "[info exists LockStates($LockName)]"
    Assert "[string compare $LockStates($LockName) Tentative] == 0 || [string compare $LockStates($LockName) Approved] == 0"

    while {[string compare $LockStates($LockName) Approved] != 0} {
	# puts "Postponing release of $LockName..."
	if {[info command mytkwait] == ""} {
	    tkwait variable LockStates($LockName)
	} else {
	    mytkwait variable LockStates($LockName) 4
	}
    }

    global HeldLocks
    Remove HeldLocks $LockName

    global LockStates
    set LockStates($LockName) Released

    # puts "ReleaseLock: Done releasing $LockName"
}


# ----------------------------------------------------------------
# Procedures called by network layer.
# ----------------------------------------------------------------

# 
# Called by network layer.  Sets LockStates(LockName) to approved or
# denied.
# 
proc NetworkResponse {LockName YesNo} {
#    puts stdout "NetworkResponse: lock = $LockName, yesno = $YesNo"

    global LockStates
    Assert "[info exists LockStates($LockName)]"
    Assert "[string compare $LockStates($LockName) Tentative] == 0"

    if {$YesNo == "Yes"} {
	set LockStates($LockName) Approved
    } else {
	global HeldLocks
	set LockStates($LockName) Denied
	Remove HeldLocks $LockName
    }
}


