#
#  This file is part of GroupKit
#
#  (c) Copyright 1992 Department of Computer Science, University of
#      Calgary, Calgary, Alberta, Canada.  All rights reserved.
#
#  Permission to use, copy, modify, and distribute this software and its
#  documentation for any purpose and without fee is hereby granted, provided
#  that the above copyright notice appears in all copies.  The University
#  of Calgary makes no representations about the suitability of this
#  software for any purpose.  It is provided "as is" without express or
#  implied warranty.
#

# Written by David Marwood

# 
# The token net layer creates a token for each lock.  When a lock is
# requested, the token is requested as well.  When the token arrives,
# the lock may be taken.  Only the token holder can get approval for
# the lock.
# 

# ----------------------------------------------------------------
# External Data
# ----------------------------------------------------------------
set NetworkLayerName "token-passing"


# ----------------------------------------------------------------
# Internal Data
# ----------------------------------------------------------------

# A keyed list of the token names and the usernum that holds them.
set TokenLocs {}
# pack append . [label .tokenlist -textvariable TokenLocs] top


# ----------------------------------------------------------------
# Creating a token.
# ----------------------------------------------------------------

# 
# Creates a token, LockName.
# 
proc NetworkCreateLock LockName {
#    puts "NetworkCreateLock:"
    global new_user
    gk_toAll RemoteNetworkCreateLock $LockName [gk_getLocalAttrib usernum]
}


# 
# RemoteNetworkCreateLock is called at all sites when a token is
# created.  It is also called when a new site joins.
# 
proc RemoteNetworkCreateLock {LockName InitUser} {
#    puts "RemoteNetworkCreateLock: LockName = $LockName, InitUser = $InitUser"
    global TokenLocs
    keylset TokenLocs $LockName $InitUser
}


# 
# By tracing gk_entrantUserNum, TokenNewUser is called at one existing site
# when a new site joins.  It creates all the tokens on the new site.
# 
trace variable gk_entrantUserNum w TokenNewUser
proc TokenNewUser args {
#    puts "TokenNewUser:"
    global LockStates
    global new_user
    global TokenLocs
    global gk_entrantUserNum
    foreach Name [array names LockStates] {
	gk_toUserNum $gk_entrantUserNum RemoteNetworkCreateLock $Name [keylget TokenLocs $Name]
    }
}


# ----------------------------------------------------------------
# Managing Lock Requests and Token Movement for the Requestor
# ----------------------------------------------------------------

# 
# If the token is here, NetworkResponse is called immediately,
# otherwise the token is requested from the holder.
# 
proc NetworkRequest LockName {
#    puts "NetworkRequest:"
    global TokenLocs
    if {[keylget TokenLocs $LockName] == [gk_getLocalAttrib usernum]} {
#	puts "NetworkRequest: Already have token."
	NetworkResponse $LockName Yes
    } else {
#	puts "NetworkRequest: Requesting token."
#	after 1000
	gk_toUserNum [keylget TokenLocs $LockName] RequestToken $LockName [gk_getLocalAttrib usernum]
    }
}


# 
# Move the token to a new location by updating TokenLocs.  The only
# reason the token would move here is because we requested it in
# NetworkRequest, so call NetworkResponse.
# 
proc NewTokenOwner {TokenName NewOwner} {
#    puts "NewTokenOwner: TokenName = $TokenName, NewOwner = $NewOwner"
    global TokenLocs
    keylset TokenLocs $TokenName $NewOwner
    if {$NewOwner == [gk_getLocalAttrib usernum]} {
	NetworkResponse $TokenName Yes
    }
}


# 
# Token holder says token is unavailable, so call NetworkResponse.
# 
proc CantHaveToken TokenName {
#    puts "CantHaveToken:"
    NetworkResponse $TokenName No
}


# 
# The site to which the request was sent doesn't hold the token.
# Likely, the token moved, but TokenLocs hasn't be updated.  Just try
# again.
# 
proc DontHaveToken TokenName {
    global TokenLocs
#    puts "DontHaveToken: resending request to user [keylget TokenLocs $TokenName]"
    gk_toUserNum [keylget TokenLocs $TokenName] RequestToken $TokenName [gk_getLocalAttrib usernum]
}


# ----------------------------------------------------------------
# Managing Lock Requests and Token Movement for the Token Holder
# ----------------------------------------------------------------

# 
# A request from a remote site for the token.  If the token is not
# here, send DontHaveToken.  If the token is unavailable, send
# CantHaveToken.  If the token is free, send NewTokenOwner to all
# sites.
# 
proc RequestToken {TokenName Usernum} {
#    puts "RequestToken: Request for $TokenName from $Usernum."
    global TokenLocs
    if {[keylget TokenLocs $TokenName] != [gk_getLocalAttrib usernum]} {
	gk_toUserNum $Usernum DontHaveToken $TokenName
#	puts "RequestToken: I don't have to token."
	return;
    }
    
    global TokenLocs
    global HeldLocks
    if {[lsearch $HeldLocks $TokenName] != -1} {
#	after 1000
#	puts "RequestToken: Sorry, but I need the token."
	gk_toUserNum $Usernum CantHaveToken $TokenName
    } else {
#	after 1000
#	puts "RequestToken: Passing the token."
	gk_toAll NewTokenOwner $TokenName $Usernum
	keylset TokenLocs $Usernum
    }
}


