# Brainstorm Conference Application
# ---------------------------------
#
#    This program is a very simple anonymous group brainstorming tool,
# where users can enter ideas and it will appear on all users' screens.
#
#    PURPOSE
#    Brainstorm demonstrates the following:
#    1. Sending messages between conferences
#    2. Updating new conference participants on the current state of the
#	conference.
#    3. Saving and loading files.
#    4. The remote scrollbars GroupKit widget.
#
#    INTERFACE
#    1. Enter ideas in the entry box by first clicking on it and then
#    	typing.
#    2. Hit RETURN or click on the 'Enter' button to put the idea in
#	the local and all the remote listboxes.
#
#    ORGANIZATION
#    Brainstorm has the following sections:
#    1. GroupKit initialization.
#    2. Interface for brainstorm.
#    3. Saving, clearing and loading routines.
#    4. Entrant updating routine.
#    5. Routine to send ideas to remote conference processes.

##########################################
# Initialize conference process as a GroupKit conference process.
gk_initConf $argv
##########################################

##########################################
# Get default GroupKit pulldown menu bar and call it .pulldownMenu.
# The last three parameters signal that we want the clear, open,
# and save menu choices.
# Next, configure as we please.  (Note that the font configuration is
# done manually at the end of this brainstorm file.  For some reason,
# gk_winConfigure would not work for fonts.)
gk_defaultMenu .pulldownMenu
gk_winConfigure .pulldownMenu -background aquamarine
gk_winConfigure .pulldownMenu -activebackground MediumAquaMarine
.pulldownMenu.file.menu add command -label "Clear" -command "Clear" -underline 0
.pulldownMenu.file.menu add command -label "Open" -command "Open" -underline 0  
.pulldownMenu.file.menu add command -label "Save" -command "Save" -underline 0 
##########################################

##########################################
# Create listbox for ideas called .ideaList.  Attach command for
# scroller.
listbox .ideaList -geometry 42x16 -background LightSteelBlue \
    -foreground DarkSlateGrey -font -Adobe-times-medium-r-normal--*-180* \
    -yscrollcommand "gk_scroller .scroll" -setgrid true
set ideaList .ideaList
##########################################

##########################################
# Create a group scrollbar by calling GroupKit procedure gk_groupScroll.
# The scrollbar returned (in .scroll) is actually a frame that contains
# both a local (.local) and a remote (.remote) scrollbar.  See gkwidgets.tcl
# for more.
gk_groupScroll .scroll -command "$ideaList yview"
.scroll.local configure -background SteelBlue -foreground LightSteelBlue \
  -activeforeground LightYellow1
.scroll.remote configure -background SteelBlue
##########################################

##########################################
# Create a frame with the entry box and 'Enter' button.  The entry box is
# called ideaBox and the button is called enterIdea.
# The callback for the entry button is putEntryInList.
frame .frame -background LightSteelBlue
entry .frame.ideaBox -textvariable myIdea -relief sunken -width 36 \
    -background aquamarine -font -Adobe-times-medium-r-normal--*-180*
button .frame.enterIdea -text Enter -command { putEntryInList } -background White \
    -activebackground ivory3 -font -Adobe-times-medium-r-normal--*-180*
##########################################

pack append .frame .frame.ideaBox left .frame.enterIdea {left}
pack append . .frame {bottom fill expand}
pack append . .pulldownMenu {top fillx} $ideaList {left fillx}
pack append . .scroll {right filly}

##########################################
# Hitting RETURN anywhere on the window calls putEntryList.
bind .frame.ideaBox <Return> "putEntryInList"
bind $ideaList <Return> "putEntryInList"
##########################################

##########################################
# Saves contents of listbox in file.  The FSBox routine asks the user which
# file to save into, the file is opened and the listbox elements are put in.
# If the file already exists, the data is APPENDED.
proc Save {} { global ideaList
    if {[$ideaList size] == 0} {return}
    set fileName [FSBox]
    if {$fileName == ""} {return}
    set fd [open $fileName a]
    for {set count 0} {$count < [$ideaList size]} {incr count 1} {
	puts $fd [$ideaList get $count]
    }
    close $fd
}
##########################################

##########################################
# Gets contents of a file and displays them, line by line, in the
# listboxes of ALL the conference processes.
proc Open {} {
    set fileName [FSBox]
    if {$fileName == ""} {return}
    Clear
    set fd [open $fileName r]
    set result [gets $fd temp]
    while {$result != -1} {
	gk_toAll doInsert $temp
	set result [gets $fd temp]
    }
    close $fd
}    
##########################################

##########################################
# Insert an idea in the listbox.
proc doInsert idea { global ideaList
    $ideaList insert end $idea
}
##########################################

##########################################
# Clears ALL the listboxes.
proc Clear {} { gk_toAll doClear }
##########################################

##########################################
# Clear a listbox.
proc doClear {} { global ideaList
    $ideaList delete 0 end
}
##########################################

##########################################
# Update the entrant with the state of the conference when he
# joins.
# The trace on gk_entrantUserNum ensures that the proc updateEntrant is
# called by ONE of the existing users when a new user joins the conference.
# (See confs.tcl for more on this.)
# updateEntrant puts all the existing ideas into the new user's listbox.

trace variable gk_entrantUserNum w {updateEntrant}

proc updateEntrant args { global gk_entrantUserNum _gk_localUser ideaList
    for {set count 0} {$count < [$ideaList size]} {incr count 1} {
	set temp [$ideaList get $count]
        gk_toUserNum $gk_entrantUserNum doInsert $temp
    }
}
##########################################

##########################################
# Put the text in myIdea into the listbox of the local and the remote
# users.
# If there is no text in myIdea, nothing is put in the listbox.
# The last line of putEntryInList is written to make sure the right lines
# in the listbox are in view.
proc putEntryInList {} { global myIdea ideaList
    if {$myIdea != ""} {
        gk_toAll doInsert $myIdea
        set myIdea ""
	if {[$ideaList size] > 16} {$ideaList yview [expr [$ideaList size]-16]}
    }
}
##########################################


##########################################
# Manual font configuration.  gk_winConfigure
# would not work for fonts on?!
.pulldownMenu.file.menu configure -font -Adobe-times-medium-r-normal--*-180*
.pulldownMenu.file configure -font -Adobe-times-medium-r-normal--*-180*
.pulldownMenu.help.menu configure -font -Adobe-times-medium-r-normal--*-180*
.pulldownMenu.help configure -font -Adobe-times-medium-r-normal--*-180*
.pulldownMenu.collaborators.menu configure -font -Adobe-times-medium-r-normal--*-180*
.pulldownMenu.collaborators configure -font -Adobe-times-medium-r-normal--*-180*
##########################################
